@@ -45,6 +45,12 @@ sub _init {
     return $self;
 }
 
+sub trace_vars {
+    my $self = shift;
+    return @_
+        ? ($self->{ TRACE_VARS } = shift)
+        :  $self->{ TRACE_VARS };
+}
 
 sub pad {
     my ($text, $pad) = @_;
@@ -64,7 +70,7 @@ sub pad {
 #------------------------------------------------------------------------
 
 sub template {
-    my ($class, $block) = @_;
+    my ($self, $block) = @_;
     $block = pad($block, 2) if $PRETTY;
 
     return "sub { return '' }" unless $block =~ /\S/;
@@ -95,7 +101,7 @@ EOF
 #------------------------------------------------------------------------
 
 sub anon_block {
-    my ($class, $block) = @_;
+    my ($self, $block) = @_;
     $block = pad($block, 2) if $PRETTY;
 
     return <<EOF;
@@ -124,7 +130,7 @@ EOF
 #------------------------------------------------------------------------
 
 sub block {
-    my ($class, $block) = @_;
+    my ($self, $block) = @_;
     return join("\n", @{ $block || [] });
 }
 
@@ -134,8 +140,8 @@ sub block {
 #------------------------------------------------------------------------
 
 sub textblock {
-    my ($class, $text) = @_;
-    return "$OUTPUT " . &text($class, $text) . ';';
+    my ($self, $text) = @_;
+    return "$OUTPUT " . &text($self, $text) . ';';
 }
 
 
@@ -144,7 +150,7 @@ sub textblock {
 #------------------------------------------------------------------------
 
 sub text {
-    my ($class, $text) = @_;
+    my ($self, $text) = @_;
     for ($text) {
         s/(["\$\@\\])/\\$1/g;
         s/\n/\\n/g;
@@ -158,7 +164,7 @@ sub text {
 #------------------------------------------------------------------------
 
 sub quoted {
-    my ($class, $items) = @_;
+    my ($self, $items) = @_;
     return '' unless @$items;
     return ("('' . " . $items->[0] . ')') if scalar @$items == 1;
     return '(' . join(' . ', @$items) . ')';
@@ -173,17 +179,40 @@ sub quoted {
 #------------------------------------------------------------------------
 
 sub ident {
-    my ($class, $ident) = @_;
+    my ($self, $ident) = @_;
     return "''" unless @$ident;
     my $ns;
 
-    # does the first element of the identifier have a NAMESPACE
-    # handler defined?
-    if (ref $class && @$ident > 2 && ($ns = $class->{ NAMESPACE })) {
-        my $key = $ident->[0];
-        $key =~ s/^'(.+)'$/$1/s;
-        if ($ns = $ns->{ $key }) {
-            return $ns->ident($ident);
+    # Careful!  Template::Parser always creates a Template::Directive object
+    # (as of v2.22_1) so $self is usually an object.  However, we used to 
+    # allow Template::Directive methods to be called as class methods and 
+    # Template::Namespace::Constants module takes advantage of this fact
+    # by calling Template::Directive->ident() when it needs to generate an
+    # identifier.  This hack guards against Mr Fuckup from coming to town
+    # when that happens.
+    
+    if (ref $self) {
+        # trace variable usage
+        if ($self->{ TRACE_VARS }) {
+            my $root = $self->{ TRACE_VARS };
+            my $n    = 0;
+            my $v;
+            while ($n < @$ident) {
+                $v = $ident->[$n];
+                for ($v) { s/^'//; s/'$// };
+                $root = $root->{ $v } ||= { };
+                $n += 2;
+            }
+        }
+
+        # does the first element of the identifier have a NAMESPACE
+        # handler defined?
+        if (@$ident > 2 && ($ns = $self->{ NAMESPACE })) {
+            my $key = $ident->[0];
+            $key =~ s/^'(.+)'$/$1/s;
+            if ($ns = $ns->{ $key }) {
+                return $ns->ident($ident);
+            }
         }
     }
         
@@ -201,7 +230,7 @@ sub ident {
 #------------------------------------------------------------------------
 
 sub identref {
-    my ($class, $ident) = @_;
+    my ($self, $ident) = @_;
     return "''" unless @$ident;
     if (scalar @$ident <= 2 && ! $ident->[1]) {
         $ident = $ident->[0];
@@ -218,7 +247,7 @@ sub identref {
 #------------------------------------------------------------------------
 
 sub assign {
-    my ($class, $var, $val, $default) = @_;
+    my ($self, $var, $val, $default) = @_;
 
     if (ref $var) {
         if (scalar @$var == 2 && ! $var->[1]) {
@@ -238,7 +267,7 @@ sub assign {
 #------------------------------------------------------------------------
 
 sub args {
-    my ($class, $args) = @_;
+    my ($self, $args) = @_;
     my $hash = shift @$args;
     push(@$args, '{ ' . join(', ', @$hash) . ' }')
         if @$hash;
@@ -252,7 +281,7 @@ sub args {
 #------------------------------------------------------------------------
 
 sub filenames {
-    my ($class, $names) = @_;
+    my ($self, $names) = @_;
     if (@$names > 1) {
         $names = '[ ' . join(', ', @$names) . ' ]';
     }
@@ -268,7 +297,7 @@ sub filenames {
 #------------------------------------------------------------------------
 
 sub get {
-    my ($class, $expr) = @_;  
+    my ($self, $expr) = @_;  
     return "$OUTPUT $expr;";
 }
 
@@ -278,7 +307,7 @@ sub get {
 #------------------------------------------------------------------------
 
 sub call {
-    my ($class, $expr) = @_;  
+    my ($self, $expr) = @_;  
     $expr .= ';';
     return $expr;
 }
@@ -289,10 +318,10 @@ sub call {
 #------------------------------------------------------------------------
 
 sub set {
-    my ($class, $setlist) = @_;
+    my ($self, $setlist) = @_;
     my $output;
     while (my ($var, $val) = splice(@$setlist, 0, 2)) {
-        $output .= &assign($class, $var, $val) . ";\n";
+        $output .= &assign($self, $var, $val) . ";\n";
     }
     chomp $output;
     return $output;
@@ -304,10 +333,10 @@ sub set {
 #------------------------------------------------------------------------
 
 sub default {
-    my ($class, $setlist) = @_;  
+    my ($self, $setlist) = @_;  
     my $output;
     while (my ($var, $val) = splice(@$setlist, 0, 2)) {
-        $output .= &assign($class, $var, $val, 1) . ";\n";
+        $output .= &assign($self, $var, $val, 1) . ";\n";
     }
     chomp $output;
     return $output;
@@ -320,9 +349,9 @@ sub default {
 #------------------------------------------------------------------------
 
 sub insert {
-    my ($class, $nameargs) = @_;
+    my ($self, $nameargs) = @_;
     my ($file, $args) = @$nameargs;
-    $file = $class->filenames($file);
+    $file = $self->filenames($file);
     return "$OUTPUT \$context->insert($file);"; 
 }
 
@@ -333,10 +362,10 @@ sub insert {
 #------------------------------------------------------------------------
 
 sub include {
-    my ($class, $nameargs) = @_;
+    my ($self, $nameargs) = @_;
     my ($file, $args) = @$nameargs;
     my $hash = shift @$args;
-    $file = $class->filenames($file);
+    $file = $self->filenames($file);
     $file .= @$hash ? ', { ' . join(', ', @$hash) . ' }' : '';
     return "$OUTPUT \$context->include($file);"; 
 }
@@ -348,10 +377,10 @@ sub include {
 #------------------------------------------------------------------------
 
 sub process {
-    my ($class, $nameargs) = @_;
+    my ($self, $nameargs) = @_;
     my ($file, $args) = @$nameargs;
     my $hash = shift @$args;
-    $file = $class->filenames($file);
+    $file = $self->filenames($file);
     $file .= @$hash ? ', { ' . join(', ', @$hash) . ' }' : '';
     return "$OUTPUT \$context->process($file);"; 
 }
@@ -366,7 +395,7 @@ sub process {
 #------------------------------------------------------------------------
 
 sub if {
-    my ($class, $expr, $block, $else) = @_;
+    my ($self, $expr, $block, $else) = @_;
     my @else = $else ? @$else : ();
     $else = pop @else;
     $block = pad($block, 1) if $PRETTY;
@@ -394,13 +423,14 @@ sub if {
 #------------------------------------------------------------------------
 
 sub foreach {
-    my ($class, $target, $list, $args, $block) = @_;
+    my ($self, $target, $list, $args, $block, $label) = @_;
     $args  = shift @$args;
     $args  = @$args ? ', { ' . join(', ', @$args) . ' }' : '';
+    $label ||= 'LOOP';
 
     my ($loop_save, $loop_set, $loop_restore, $setiter);
     if ($target) {
-        $loop_save    = 'eval { $_tt_oldloop = ' . &ident($class, ["'loop'"]) . ' }';
+        $loop_save    = 'eval { $_tt_oldloop = ' . &ident($self, ["'loop'"]) . ' }';
         $loop_set     = "\$stash->{'$target'} = \$_tt_value";
         $loop_restore = "\$stash->set('loop', \$_tt_oldloop)";
     }
@@ -430,7 +460,7 @@ do {
     $loop_save;
     \$stash->set('loop', \$_tt_list);
     eval {
-LOOP:   while (! \$_tt_error) {
+$label:   while (! \$_tt_error) {
             $loop_set;
 $block;
             (\$_tt_value, \$_tt_error) = \$_tt_list->get_next();
@@ -451,9 +481,11 @@ EOF
 #------------------------------------------------------------------------
 
 sub next {
+    my ($self, $label) = @_;
+    $label ||= 'LOOP';
     return <<EOF;
 (\$_tt_value, \$_tt_error) = \$_tt_list->get_next();
-next LOOP;
+next $label;
 EOF
 }
 
@@ -464,14 +496,14 @@ EOF
 #------------------------------------------------------------------------
 
 sub wrapper {
-    my ($class, $nameargs, $block) = @_;
+    my ($self, $nameargs, $block) = @_;
     my ($file, $args) = @$nameargs;
     my $hash = shift @$args;
 
     local $" = ', ';
 #    print STDERR "wrapper([@$file], { @$hash })\n";
 
-    return $class->multi_wrapper($file, $hash, $block)
+    return $self->multi_wrapper($file, $hash, $block)
         if @$file > 1;
     $file = shift @$file;
 
@@ -492,7 +524,7 @@ EOF
 
 
 sub multi_wrapper {
-    my ($class, $file, $hash, $block) = @_;
+    my ($self, $file, $hash, $block) = @_;
     $block = pad($block, 1) if $PRETTY;
 
     push(@$hash, "'content'", '$output');
@@ -523,15 +555,16 @@ EOF
 #------------------------------------------------------------------------
 
 sub while {
-    my ($class, $expr, $block) = @_;
+    my ($self, $expr, $block, $label) = @_;
     $block = pad($block, 2) if $PRETTY;
+    $label ||= 'LOOP';
 
     return <<EOF;
 
 # WHILE
 do {
     my \$_tt_failsafe = $WHILE_MAX;
-LOOP:
+$label:
     while (--\$_tt_failsafe && ($expr)) {
 $block
     }
@@ -550,7 +583,7 @@ EOF
 #------------------------------------------------------------------------
 
 sub switch {
-    my ($class, $expr, $case) = @_;
+    my ($self, $expr, $case) = @_;
     my @case = @$case;
     my ($match, $block, $default);
     my $caseblock = '';
@@ -598,7 +631,7 @@ EOF
 #------------------------------------------------------------------------
 
 sub try {
-    my ($class, $block, $catch) = @_;
+    my ($self, $block, $catch) = @_;
     my @catch = @$catch;
     my ($match, $mblock, $default, $final, $n);
     my $catchblock = '';
@@ -666,7 +699,7 @@ EOF
 #------------------------------------------------------------------------
 
 sub throw {
-    my ($class, $nameargs) = @_;
+    my ($self, $nameargs) = @_;
     my ($type, $args) = @$nameargs;
     my $hash = shift(@$args);
     my $info = shift(@$args);
@@ -712,7 +745,7 @@ sub clear {
 # NOTE: this is redundant, being hard-coded (for now) into Parser.yp
 #------------------------------------------------------------------------
 
-sub break {
+sub OLD_break {
     return 'last LOOP;';
 }
 
@@ -739,13 +772,13 @@ sub stop {
 #------------------------------------------------------------------------
 
 sub use {
-    my ($class, $lnameargs) = @_;
+    my ($self, $lnameargs) = @_;
     my ($file, $args, $alias) = @$lnameargs;
     $file = shift @$file;       # same production rule as INCLUDE
     $alias ||= $file;
-    $args = &args($class, $args);
+    $args = &args($self, $args);
     $file .= ", $args" if $args;
-#    my $set = &assign($class, $alias, '$plugin'); 
+#    my $set = &assign($self, $alias, '$plugin'); 
     return "# USE\n"
          . "\$stash->set($alias,\n"
          . "            \$context->plugin($file));";
@@ -757,7 +790,7 @@ sub use {
 #------------------------------------------------------------------------
 
 sub view {
-    my ($class, $nameargs, $block, $defblocks) = @_;
+    my ($self, $nameargs, $block, $defblocks) = @_;
     my ($name, $args) = @$nameargs;
     my $hash = shift @$args;
     $name = shift @$name;       # same production rule as INCLUDE
@@ -796,7 +829,7 @@ EOF
 #------------------------------------------------------------------------
 
 sub perl {
-    my ($class, $block) = @_;
+    my ($self, $block) = @_;
     $block = pad($block, 1) if $PRETTY;
 
     return <<EOF;
@@ -831,7 +864,7 @@ EOF
 #------------------------------------------------------------------------
 
 sub no_perl {
-    my $class = shift;
+    my $self = shift;
     return "\$context->throw('perl', 'EVAL_PERL not set');";
 }
 
@@ -844,7 +877,7 @@ sub no_perl {
 #------------------------------------------------------------------------
 
 sub rawperl {
-    my ($class, $block, $line) = @_;
+    my ($self, $block, $line) = @_;
     for ($block) {
         s/^\n+//;
         s/\n+$//;
@@ -866,10 +899,10 @@ EOF
 #------------------------------------------------------------------------
 
 sub filter {
-    my ($class, $lnameargs, $block) = @_;
+    my ($self, $lnameargs, $block) = @_;
     my ($name, $args, $alias) = @$lnameargs;
     $name = shift @$name;
-    $args = &args($class, $args);
+    $args = &args($self, $args);
     $args = $args ? "$args, $alias" : ", undef, $alias"
         if $alias;
     $name .= ", $args" if $args;
@@ -896,7 +929,7 @@ EOF
 #------------------------------------------------------------------------
 
 sub capture {
-    my ($class, $name, $block) = @_;
+    my ($self, $name, $block) = @_;
 
     if (ref $name) {
         if (scalar @$name == 2 && ! $name->[1]) {
@@ -926,7 +959,7 @@ EOF
 #------------------------------------------------------------------------
 
 sub macro {
-    my ($class, $ident, $block, $args) = @_;
+    my ($self, $ident, $block, $args) = @_;
     $block = pad($block, 2) if $PRETTY;
 
     if ($args) {
@@ -980,7 +1013,7 @@ EOF
 
 
 sub debug {
-    my ($class, $nameargs) = @_;
+    my ($self, $nameargs) = @_;
     my ($file, $args) = @$nameargs;
     my $hash = shift @$args;
     $args  = join(', ', @$file, @$args);
