@@ -26,8 +26,10 @@ use base 'Template::Base';
 use Template::Constants;
 use Scalar::Util 'blessed';
 
-our $VERSION   = 2.86;
-our $AVAILABLE = { };
+our $VERSION         = 2.87;
+our $AVAILABLE       = { };
+our $TRUNCATE_LENGTH = 32;
+our $TRUNCATE_ADDON  = '...';
 
 
 #------------------------------------------------------------------------
@@ -531,13 +533,22 @@ sub remove_filter_factory {
 
 sub truncate_filter_factory {
     my ($context, $len, $char) = @_;
-    $len = 32 unless defined $len;
-    $char = "..." unless defined $char;
+    $len  = $TRUNCATE_LENGTH unless defined $len;
+    $char = $TRUNCATE_ADDON  unless defined $char;
+
+    # Length of char is the minimum length
+    my $lchar = length $char;
+    if ($len < $lchar) {
+        $char  = substr($char, 0, $len);
+        $lchar = $len;
+    }
 
     return sub {
         my $text = shift;
         return $text if length $text <= $len;
-        return substr($text, 0, $len - length($char)) . $char;
+        return substr($text, 0, $len - $lchar) . $char;
+
+
     }
 }
 
