@@ -9,14 +9,11 @@
 #   Andy Wardley   <abw@wardley.org>
 #
 # COPYRIGHT
-#   Copyright (C) 2001-2006 Andy Wardley.  All Rights Reserved.
+#   Copyright (C) 2001-2007 Andy Wardley.  All Rights Reserved.
 #
 #   This module is free software; you can redistribute it and/or
 #   modify it under the same terms as Perl itself.
 #
-# REVISION
-#   $Id: String.pm,v 2.40 2006/05/30 17:01:36 abw Exp $
-#
 #============================================================================
 
 package Template::Plugin::String;
@@ -46,23 +43,23 @@ sub new {
     $class = ref($class) || $class;
 
     my $text = defined $config->{ text } 
-	? $config->{ text }
+        ? $config->{ text }
         : (@args ? shift(@args) : '');
 
 #    print STDERR "text: [$text]\n";
 #    print STDERR "class: [$class]\n";
     
     my $self = bless {
-	text     => $text,
-	filters  => [ ],
-	_CONTEXT => $context,
+        text     => $text,
+        filters  => [ ],
+        _CONTEXT => $context,
     }, $class;
 
     my $filter = $config->{ filter } || $config->{ filters };
 
     # install any output filters specified as 'filter' or 'filters' option
     $self->output_filter($filter)
-	if $filter;
+        if $filter;
 
     return $self;
 }
@@ -76,10 +73,10 @@ sub text {
     my $context = $self->{ _CONTEXT };
 
     foreach my $dispatch (@{ $self->{ filters } }) {
-	my ($name, $args) = @$dispatch;
-	my $code = $context->filter($name, $args)
-	    || $self->throw($context->error());
-	$text = &$code($text);
+        my ($name, $args) = @$dispatch;
+        my $code = $context->filter($name, $args)
+            || $self->throw($context->error());
+        $text = &$code($text);
     }
     return $text;
 }
@@ -113,33 +110,33 @@ sub output_filter {
     my $count = 0;
 
     if (ref $filter eq 'HASH') {
-	$filter = [ %$filter ];
+        $filter = [ %$filter ];
     }
     elsif (ref $filter ne 'ARRAY') {
-	$filter = [ split(/\s*\W+\s*/, $filter) ];
+        $filter = [ split(/\s*\W+\s*/, $filter) ];
     }
 
     while (@$filter) {
-	$name = shift @$filter;
+        $name = shift @$filter;
 
-	# args may follow as a reference (or empty string, e.g. { foo => '' }
-	if (@$filter && (ref($filter->[0]) || ! length $filter->[0])) {
-	    $args = shift @$filter;
-	    if ($args) {
-		$args = [ $args ] unless ref $args eq 'ARRAY';
-	    }
-	    else {
-		$args = [ ];
-	    }
-	}
-	else {
-	    $args = [ ];
-	}
+        # args may follow as a reference (or empty string, e.g. { foo => '' }
+        if (@$filter && (ref($filter->[0]) || ! length $filter->[0])) {
+            $args = shift @$filter;
+            if ($args) {
+                $args = [ $args ] unless ref $args eq 'ARRAY';
+            }
+            else {
+                $args = [ ];
+            }
+        }
+        else {
+            $args = [ ];
+        }
 
-#	$self->DEBUG("adding output filter $name(@$args)\n");
+#       $self->DEBUG("adding output filter $name(@$args)\n");
 
-	push(@$filters, [ $name, $args ]);
-	$count++;
+        push(@$filters, [ $name, $args ]);
+        $count++;
     }
 
     return '';
@@ -186,9 +183,9 @@ sub center {
     $width ||= 0;
 
     if ($len < $width) {
-	my $lpad = int(($width - $len) / 2);
-	my $rpad = $width - $len - $lpad;
-	$self->{ text } = (' ' x $lpad) . $self->{ text } . (' ' x $rpad);
+        my $lpad = int(($width - $len) / 2);
+        my $rpad = $width - $len - $lpad;
+        $self->{ text } = (' ' x $lpad) . $self->{ text } . (' ' x $rpad);
     }
 
     return $self;
@@ -201,7 +198,7 @@ sub left {
     $width ||= 0;
 
     $self->{ text } .= (' ' x ($width - $len))
-	if $width > $len;
+        if $width > $len;
 
     return $self;
 }
@@ -213,7 +210,7 @@ sub right {
     $width ||= 0;
 
     $self->{ text } = (' ' x ($width - $len)) . $self->{ text }
-	if $width > $len;
+        if $width > $len;
 
     return $self;
 }
@@ -233,7 +230,7 @@ sub filter {
     my $context = $self->{ _CONTEXT };
 
     my $code = $context->filter($name, \@args)
-	|| $self->throw($context->error());
+        || $self->throw($context->error());
     return &$code($self->{ text });
 }
 
@@ -279,8 +276,8 @@ sub chomp {
 sub trim {
     my $self = CORE::shift;
     for ($self->{ text }) {
-	s/^\s+//; 
-	s/\s+$//; 
+        s/^\s+//; 
+        s/\s+$//; 
     }
     return $self;    
 }
@@ -289,9 +286,9 @@ sub trim {
 sub collapse {
     my $self = CORE::shift;
     for ($self->{ text }) {
-	s/^\s+//; 
-	s/\s+$//; 
-	s/\s+/ /g 
+        s/^\s+//; 
+        s/\s+$//; 
+        s/\s+/ /g 
     }
     return $self;    
 
@@ -311,7 +308,7 @@ sub truncate {
     $suffix ||= '';
     return $self if CORE::length $self->{ text } <= $length;
     $self->{ text } = CORE::substr($self->{ text }, 0, 
-			     $length - CORE::length($suffix)) . $suffix;
+                             $length - CORE::length($suffix)) . $suffix;
     return $self;
 }
 
@@ -386,18 +383,6 @@ sub equals {
 
 __END__
 
-
-#------------------------------------------------------------------------
-# IMPORTANT NOTE
-#   This documentation is generated automatically from source
-#   templates.  Any changes you make here may be lost.
-# 
-#   The 'docsrc' documentation source bundle is available for download
-#   from http://www.template-toolkit.org/docs.html and contains all
-#   the source templates, XML files, scripts, etc., from which the
-#   documentation for the Template Toolkit is built.
-#------------------------------------------------------------------------
-
 =head1 NAME
 
 Template::Plugin::String - Object oriented interface for string manipulation
@@ -430,50 +415,50 @@ Template::Plugin::String - Object oriented interface for string manipulation
 
 =head1 DESCRIPTION
 
-This module implements a String class for doing stringy things to
+This module implements a C<String> class for doing stringy things to
 text in an object-oriented way. 
 
-You can create a String object via the USE directive, adding any 
-initial text value as an argument or as the named parameter 'text'.
+You can create a C<String> object via the C<USE> directive, adding any 
+initial text value as an argument or as the named parameter C<text>.
 
     [% USE String %]
     [% USE String 'initial text' %]
     [% USE String text='initial text' %]
 
-The object created will be referenced as 'String' by default, but you
+The object created will be referenced as C<String> by default, but you
 can provide a different variable name for the object to be assigned
 to:
 
     [% USE greeting = String 'Hello World' %]
 
-Once you've got a String object, you can use it as a prototype to 
-create other String objects with the new() method.
+Once you've got a C<String> object, you can use it as a prototype to 
+create other C<String> objects with the C<new()> method.
 
     [% USE String %]
     [% greeting = String.new('Hello World') %]
 
-The new() method also accepts an initial text string as an argument
-or the named parameter 'text'.
+The C<new()> method also accepts an initial text string as an argument
+or the named parameter C<text>.
 
     [% greeting = String.new( text => 'Hello World' ) %]
 
-You can also call copy() to create a new String as a copy of the 
+You can also call C<copy()> to create a new C<String> as a copy of the 
 original.
 
     [% greet2 = greeting.copy %]
 
-The String object has a text() method to return the content of the 
+The C<String> object has a C<text()> method to return the content of the 
 string.
 
     [% greeting.text %]
 
 However, it is sufficient to simply print the string and let the
-overloaded stringification operator call the text() method
+overloaded stringification operator call the C<text()> method
 automatically for you.
 
     [% greeting %]
 
-Thus, you can treat String objects pretty much like any regular piece
+Thus, you can treat C<String> objects pretty much like any regular piece
 of text, interpolating it into other strings, for example:
 
     [% msg = "It printed '$greeting' and then dumped core\n" %]
@@ -489,55 +474,46 @@ take a copy first:
 
     [% msg.copy.append("PS  Don't eat the yellow snow") %]
 
-These methods return a reference to the String object itself.  This
+These methods return a reference to the C<String> object itself.  This
 allows you to chain multiple methods together.
 
     [% msg.copy.append('foo').right(72) %]
 
-It also means that in the above examples, the String is returned which
-causes the text() method to be called, which results in the new value of
+It also means that in the above examples, the C<String> is returned which
+causes the C<text()> method to be called, which results in the new value of
 the string being printed.  To suppress printing of the string, you can
-use the CALL directive.
+use the C<CALL> directive.
 
     [% foo = String.new('foo') %]
-
+    
     [% foo.append('bar') %]         # prints "foobar"
-
+    
     [% CALL foo.append('bar') %]    # nothing
 
-=head1 METHODS
+=head1 CONSTRUCTOR METHODS
 
-=head2 Construction Methods
+These methods are used to create new C<String> objects.
 
-The following methods are used to create new String objects.
-
-=over 4
-
-=item new()
+=head2 new()
 
 Creates a new string using an initial value passed as a positional
-argument or the named parameter 'text'.
+argument or the named parameter C<text>.
 
     [% USE String %]
     [% msg = String.new('Hello World') %]
     [% msg = String.new( text => 'Hello World' ) %]
 
-=item copy()
+=head2 copy()
 
-Creates a new String object which contains a copy of the original string.
+Creates a new C<String> object which contains a copy of the original string.
 
     [% msg2 = msg.copy %]
 
-=back
+=head1 INSPECTOR METHODS
 
-=head2 Inspection Methods
+These methods are used to examine the string.
 
-These methods are used to inspect the string content or other parameters
-relevant to the string.
-
-=over 4
-
-=item text()
+=head2 text()
 
 Returns the internal text value of the string.  The stringification
 operator is overloaded to call this method.  Thus the following are
@@ -546,27 +522,25 @@ equivalent:
     [% msg.text %]
     [% msg %]
 
-=item length()
+=head2 length()
 
 Returns the length of the string.
 
     [% USE String("foo") %]
-
     [% String.length %]   # => 3
 
-=item search($pattern)
+=head2 search($pattern)
 
-Searches the string for the regular expression specified in $pattern
+Searches the string for the regular expression specified in C<$pattern>
 returning true if found or false otherwise.
 
     [% item = String.new('foo bar baz wiz waz woz') %]
-
     [% item.search('wiz') ? 'WIZZY! :-)' : 'not wizzy :-(' %]
 
-=item split($pattern, $limit)
+=head2 split($pattern, $limit)
 
-Splits the string based on the delimiter $pattern and optional $limit.  
-Delegates to Perl's internal split() so the parameters are exactly the same.
+Splits the string based on the delimiter C<$pattern> and optional C<$limit>.
+Delegates to Perl's internal C<split()> so the parameters are exactly the same.
 
     [% FOREACH item.split %]
          ...
@@ -576,55 +550,32 @@ Delegates to Perl's internal split() so the parameters are exactly the same.
          ...
     [% END %]
 
-=item substr($offset, $length, $replacement)
-
-Returns a substring starting at $offset, for $length characters.
-
-    [% str = String.new('foo bar baz wiz waz woz') %]
-    [% str.substr(4, 3) %]    # bar
-
-If $length is not specified then it returns everything from the $offset
-to the end of the string.
-
-    [% str.substr(12) %]      # wiz waz woz
-
-If both $length and $replacement are specified, then the method
-replaces everything from $offset for $length characters with
-$replacement.  The substring removed from the string is then 
-returned.
-
-    [% str.substr(0, 11, 'FOO') %]   # foo bar baz
-    [% str %]                        # FOO wiz waz woz
-
-=back
-
-=head2 Mutation Methods
+=head1 MUTATOR METHODS
 
 These methods modify the internal value of the string.  For example:
 
     [% USE str=String('foobar') %]
+    [% str.append('.html') %]   # str => 'foobar.html'
 
-    [% str.append('.html') %]	# str => 'foobar.html'
-
-The value of the String 'str' is now 'foobar.html'.  If you don't want
+The value of C<str> is now 'C<foobar.html>'.  If you don't want
 to modify the string then simply take a copy first.
 
     [% str.copy.append('.html') %]
 
-These methods all return a reference to the String object itself.  This
+These methods all return a reference to the C<String> object itself.  This
 has two important benefits.  The first is that when used as above, the 
-String object 'str' returned by the append() method will be stringified
-with a call to its text() method.  This will return the newly modified 
+C<String> object 'C<str>' returned by the C<append()> method will be stringified
+with a call to its C<text()> method.  This will return the newly modified 
 string content.  In other words, a directive like:
 
     [% str.append('.html') %]
 
 will update the string and also print the new value.  If you just want
-to update the string but not print the new value then use CALL.
+to update the string but not print the new value then use C<CALL>.
 
     [% CALL str.append('.html') %]
 
-The other benefit of these methods returning a reference to the String
+The other benefit of these methods returning a reference to the C<String>
 is that you can chain as many different method calls together as you
 like.  For example:
 
@@ -632,89 +583,84 @@ like.  For example:
 
 Here are the methods:
 
-=over 4
-
-=item push($suffix, ...) / append($suffix, ...)
+=head2 push($suffix, ...) / append($suffix, ...)
 
 Appends all arguments to the end of the string.  The 
-append() method is provided as an alias for push().
+C<append()> method is provided as an alias for C<push()>.
 
     [% msg.push('foo', 'bar') %]
     [% msg.append('foo', 'bar') %]
 
-=item pop($suffix)
+=head2 pop($suffix)
 
 Removes the suffix passed as an argument from the end of the String.
 
     [% USE String 'foo bar' %]
     [% String.pop(' bar')   %]   # => 'foo'
 
-=item unshift($prefix, ...) / prepend($prefix, ...)
+=head2 unshift($prefix, ...) / prepend($prefix, ...)
 
 Prepends all arguments to the beginning of the string.  The
-prepend() method is provided as an alias for unshift().
+C<prepend()> method is provided as an alias for C<unshift()>.
 
     [% msg.unshift('foo ', 'bar ') %]
     [% msg.prepend('foo ', 'bar ') %]
 
-=item shift($prefix)
+=head2 shift($prefix)
 
 Removes the prefix passed as an argument from the start of the String.
 
     [% USE String 'foo bar' %]
     [% String.shift('foo ') %]   # => 'bar'
 
-=item left($pad)
+=head2 left($pad)
 
-If the length of the string is less than $pad then the string is left
-formatted and padded with spaces to $pad length.
+If the length of the string is less than C<$pad> then the string is left
+formatted and padded with spaces to C<$pad> length.
 
     [% msg.left(20) %]
 
-=item right($pad)
+=head2 right($pad)
 
-As per left() but right padding the String to a length of $pad.
+As per L<left()> but right padding the C<String> to a length of C<$pad>.
 
     [% msg.right(20) %]
 
-=item center($pad) / centre($pad)
+=head2 center($pad) / centre($pad)
 
-As per left() and right() but formatting the String to be centered within 
-a space padded string of length $pad.  The centre() method is provided as 
-an alias for center() to keep Yanks and Limeys happy.
+As per L<left()> and L<right()> but formatting the C<String> to be centered within 
+a space padded string of length C<$pad>.  The C<centre()> method is provided as 
+an alias for C<center()>.
 
     [% msg.center(20) %]    # American spelling
     [% msg.centre(20) %]    # European spelling
 
-=item format($format)
+=head2 format($format)
 
-Apply a format in the style of sprintf() to the string.
+Apply a format in the style of C<sprintf()> to the string.
 
     [% USE String("world") %]
     [% String.format("Hello %s\n") %]  # => "Hello World\n"
 
-=item upper()
+=head2 upper()
 
 Converts the string to upper case.
 
     [% USE String("foo") %]
-
     [% String.upper %]  # => 'FOO'
 
-=item lower()
+=head2 lower()
 
 Converts the string to lower case
 
     [% USE String("FOO") %]
-
     [% String.lower %]  # => 'foo'
 
-=item capital()
+=head2 capital()
 
 Converts the first character of the string to upper case.  
 
     [% USE String("foo") %]
-
     [% String.capital %]  # => 'Foo'
 
 The remainder of the string is left untouched.  To force the string to
@@ -722,105 +668,87 @@ be all lower case with only the first letter capitalised, you can do
 something like this:
 
     [% USE String("FOO") %]
-
     [% String.lower.capital %]  # => 'Foo'
 
-=item chop()
+=head2 chop()
 
 Removes the last character from the string.
 
     [% USE String("foop") %]
+    [% String.chop %]   # => 'foo'
 
-    [% String.chop %]	# => 'foo'
-
-=item chomp()
+=head2 chomp()
 
 Removes the trailing newline from the string.
 
     [% USE String("foo\n") %]
+    [% String.chomp %]  # => 'foo'
 
-    [% String.chomp %]	# => 'foo'
-
-=item trim()
+=head2 trim()
 
 Removes all leading and trailing whitespace from the string
 
     [% USE String("   foo   \n\n ") %]
+    [% String.trim %]   # => 'foo'
 
-    [% String.trim %]	# => 'foo'
-
-=item collapse()
+=head2 collapse()
 
 Removes all leading and trailing whitespace and collapses any sequences
 of multiple whitespace to a single space.
 
     [% USE String(" \n\r  \t  foo   \n \n bar  \n") %]
-
     [% String.collapse %]   # => "foo bar"
 
-=item truncate($length, $suffix)
+=head2 truncate($length, $suffix)
 
-Truncates the string to $length characters.
+Truncates the string to C<$length> characters.
 
     [% USE String('long string') %]
     [% String.truncate(4) %]  # => 'long'
 
-If $suffix is specified then it will be appended to the truncated
+If C<$suffix> is specified then it will be appended to the truncated
 string.  In this case, the string will be further shortened by the 
 length of the suffix to ensure that the newly constructed string
-complete with suffix is exactly $length characters long.
+complete with suffix is exactly C<$length> characters long.
 
     [% USE msg = String('Hello World') %]
     [% msg.truncate(8, '...') %]   # => 'Hello...'
 
-=item replace($search, $replace)
+=head2 replace($search, $replace)
 
-Replaces all occurences of $search in the string with $replace.
+Replaces all occurences of C<$search> in the string with C<$replace>.
 
     [% USE String('foo bar foo baz') %]
     [% String.replace('foo', 'wiz')  %]  # => 'wiz bar wiz baz'
 
-=item remove($search)
+=head2 remove($search)
 
-Remove all occurences of $search in the string.
+Remove all occurences of C<$search> in the string.
 
     [% USE String('foo bar foo baz') %]
     [% String.remove('foo ')  %]  # => 'bar baz'
 
-=item repeat($count)
+=head2 repeat($count)
 
-Repeats the string $count times.
+Repeats the string C<$count> times.
 
     [% USE String('foo ') %]
     [% String.repeat(3)  %]  # => 'foo foo foo '
 
-=back
-
 =head1 AUTHOR
 
-Andy Wardley E<lt>abw@wardley.orgE<gt>
-
-L<http://wardley.org/|http://wardley.org/>
-
-
-
-
-=head1 VERSION
-
-2.4, distributed as part of the
-Template Toolkit version 2.18, released on 09 February 2007.
+Andy Wardley E<lt>abw@wardley.orgE<gt> L<http://wardley.org/>
 
 =head1 COPYRIGHT
 
-  Copyright (C) 1996-2007 Andy Wardley.  All Rights Reserved.
-
+Copyright (C) 1996-2007 Andy Wardley.  All Rights Reserved.
 
 This module is free software; you can redistribute it and/or
 modify it under the same terms as Perl itself.
 
 =head1 SEE ALSO
 
-L<Template::Plugin|Template::Plugin>
+L<Template::Plugin>
 
 =cut
 
