@@ -21,9 +21,6 @@
 #    work-around is to do [% view.print(hash1); view.print(hash2); 
 #    view.print(hash3) %] or [% view.print(hash1, hash2, hash3, { }) %]
 #
-# REVISION
-#   $Id: View.pm 1022 2006-05-30 17:01:43Z abw $
-#
 #============================================================================
 
 package Template::View;
@@ -64,8 +61,8 @@ sub _init {
     my $map = $config->{ map } || { };
     $map->{ default } = $config->{ default } unless defined $map->{ default };
     $self->{ map } = {
-	%$MAP,
-	%$map,
+        %$MAP,
+        %$map,
     };
 
     # local BLOCKs definition table
@@ -73,7 +70,7 @@ sub _init {
     
     # name of presentation method which printed objects might provide
     $self->{ method } = defined $config->{ method } 
-	                      ? $config->{ method } : 'present';
+                              ? $config->{ method } : 'present';
     
     # view is sealed by default preventing variable update after 
     # definition, however we don't actually seal a view until the 
@@ -84,9 +81,13 @@ sub _init {
 
     # copy remaining config items from $config or set defaults
     foreach my $arg (qw( base prefix suffix notfound silent )) {
-	$self->{ $arg } = $config->{ $arg } || '';
+        $self->{ $arg } = $config->{ $arg } || '';
     }
 
+    # check that any base specified is defined
+    return $self->error('Invalid base specified for view')
+        if exists $config->{ base } && ! $self->{ base };
+
     # name of data item used by view()
     $self->{ item } = $config->{ item } || 'item';
 
@@ -94,7 +95,7 @@ sub _init {
     $self->{ include_prefix } = $config->{ include_prefix } || 'include_';
     # what about mapping foobar() to include('foobar')?
     $self->{ include_naked  } = defined $config->{ include_naked } 
-			              ? $config->{ include_naked } : 1;
+                                      ? $config->{ include_naked } : 1;
 
     # map methods of form ${view_prefix}_foobar() to include('foobar')?
     $self->{ view_prefix } = $config->{ view_prefix } || 'view_';
@@ -105,10 +106,10 @@ sub _init {
     # view template to create data items via the AUTOLOAD; once sealed via
     # call to seal(), the AUTOLOAD will not update any internal items.
     delete @$config{ qw( base method map default prefix suffix notfound item 
-			 include_prefix include_naked silent sealed
-			 view_prefix view_naked blocks ) };
+                         include_prefix include_naked silent sealed
+                         view_prefix view_naked blocks ) };
     $config = { %{ $self->{ base }->{ data } }, %$config }
-	if $self->{ base };
+        if $self->{ base };
     $self->{ data   } = $config;
     $self->{ SEALED } = 0;
 
@@ -153,8 +154,8 @@ sub clone {
 
     # merge maps
     $clone->{ map } = {
-	%{ $self->{ map } },
-	%{ $config->{ map } || { } },
+        %{ $self->{ map } },
+        %{ $config->{ map } || { } },
     };
 
     # "map => { default=>'xxx' }" can be specified as "default => 'xxx'"
@@ -163,9 +164,9 @@ sub clone {
 
     # update any remaining config items
     my @args = qw( base prefix suffix notfound item method include_prefix 
-		   include_naked view_prefix view_naked );
+                   include_naked view_prefix view_naked );
     foreach my $arg (@args) {
-	$clone->{ $arg } = $config->{ $arg } if defined $config->{ $arg };
+        $clone->{ $arg } = $config->{ $arg } if defined $config->{ $arg };
     }
     push(@args, qw( default map ));
     delete @$config{ @args };
@@ -224,14 +225,14 @@ sub print {
             # no specific map entry for object, maybe it implements a 
             # 'present' (or other) method?
             if ( $method && UNIVERSAL::can($item, $method) ) {
-                $present = $item->$method($self);	## call item method
+                $present = $item->$method($self);       ## call item method
                 # undef returned indicates error, note that we expect 
                 # $item to have called error() on the view
                 return unless defined $present;
                 $output .= $present;
-                next;					## NEXT
+                next;                                   ## NEXT
             }   
-            elsif ( UNIVERSAL::isa($item, 'HASH' ) 
+            elsif ( ref($item) eq 'HASH' 
                     && defined($newtype = $item->{$method})
                     && defined($template = $map->{"$method=>$newtype"})) {
             }
@@ -244,9 +245,9 @@ sub print {
                 ($template = $type) =~ s/\W+/_/g;
             }
         }
-#	else {
-#	    $self->DEBUG("defined map type for $type: $template\n");
-#	}
+#       else {
+#           $self->DEBUG("defined map type for $type: $template\n");
+#       }
         $self->DEBUG("printing view '", $template || '', "', $item\n") if $DEBUG;
         $output .= $self->view($template, $item)
             if $template;
@@ -316,15 +317,15 @@ sub template {
     my ($self, $name) = @_;
     my $context = $self->{ _CONTEXT };
     return $context->throw(Template::Constants::ERROR_VIEW,
-			   "no view template specified")
-	unless $name;
+                           "no view template specified")
+        unless $name;
 
     my $notfound = $self->{ notfound };
     my $base = $self->{ base };
     my ($template, $block, $error);
 
     return $block
-	if ($block = $self->{ _BLOCKS }->{ $name });
+        if ($block = $self->{ _BLOCKS }->{ $name });
     
     # try the named template
     $template = $self->template_name($name);
@@ -333,25 +334,25 @@ sub template {
 
     # try asking the base view if not found
     if (($error = $@) && $base) {
-	$self->DEBUG("asking base for $name\n") if $DEBUG;
-	eval { $template = $base->template($name) };
+        $self->DEBUG("asking base for $name\n") if $DEBUG;
+        eval { $template = $base->template($name) };
     }
 
     # try the 'notfound' template (if defined) if that failed
     if (($error = $@) && $notfound) {
-	unless ($template = $self->{ _BLOCKS }->{ $notfound }) {
-	    $notfound = $self->template_name($notfound);
-	    $self->DEBUG("not found, looking for $notfound\n") if $DEBUG;
-	    eval { $template = $context->template($notfound) };
+        unless ($template = $self->{ _BLOCKS }->{ $notfound }) {
+            $notfound = $self->template_name($notfound);
+            $self->DEBUG("not found, looking for $notfound\n") if $DEBUG;
+            eval { $template = $context->template($notfound) };
 
-	    return $context->throw(Template::Constants::ERROR_VIEW, $error)
-		if $@;	# return first error
-	}
+            return $context->throw(Template::Constants::ERROR_VIEW, $error)
+                if $@;  # return first error
+        }
     }
     elsif ($error) {
-	$self->DEBUG("no 'notfound'\n") 
-	    if $DEBUG;
-	return $context->throw(Template::Constants::ERROR_VIEW, $error);
+        $self->DEBUG("no 'notfound'\n") 
+            if $DEBUG;
+        return $context->throw(Template::Constants::ERROR_VIEW, $error);
     }
     return $template;
 }
@@ -367,7 +368,7 @@ sub template {
 sub template_name {
     my ($self, $template) = @_;
     $template = $self->{ prefix } . $template . $self->{ suffix }
-	if $template;
+        if $template;
 
     $self->DEBUG("template name: $template\n") if $DEBUG;
     return $template;
@@ -384,7 +385,7 @@ sub template_name {
 sub default {
     my $self = shift;
     return @_ ? ($self->{ map }->{ default } = shift) 
-	      :  $self->{ map }->{ default };
+              :  $self->{ map }->{ default };
 }
 
 
@@ -411,55 +412,55 @@ sub AUTOLOAD {
     return if $item eq 'DESTROY';
 
     if ($item =~ /^[\._]/) {
-	return $self->{ _CONTEXT }->throw(Template::Constants::ERROR_VIEW,
-			    "attempt to view private member: $item");
+        return $self->{ _CONTEXT }->throw(Template::Constants::ERROR_VIEW,
+                            "attempt to view private member: $item");
     }
     elsif (exists $self->{ $item }) {
-	# update existing config item (e.g. 'prefix') if unsealed
-	return $self->{ _CONTEXT }->throw(Template::Constants::ERROR_VIEW,
-			    "cannot update config item in sealed view: $item")
-	    if @_ && $self->{ SEALED };
-	$self->DEBUG("accessing item: $item\n") if $DEBUG;
-	return @_ ? ($self->{ $item } = shift) : $self->{ $item };
+        # update existing config item (e.g. 'prefix') if unsealed
+        return $self->{ _CONTEXT }->throw(Template::Constants::ERROR_VIEW,
+                            "cannot update config item in sealed view: $item")
+            if @_ && $self->{ SEALED };
+        $self->DEBUG("accessing item: $item\n") if $DEBUG;
+        return @_ ? ($self->{ $item } = shift) : $self->{ $item };
     }
     elsif (exists $self->{ data }->{ $item }) {
-	# get/update existing data item (must be unsealed to update)
-	if (@_ && $self->{ SEALED }) {
-	    return $self->{ _CONTEXT }->throw(Template::Constants::ERROR_VIEW,
-				  "cannot update item in sealed view: $item")
-		unless $self->{ silent };
-	    # ignore args if silent
-	    @_ = ();
-	}
-	$self->DEBUG(@_ ? "updating data item: $item <= $_[0]\n" 
-		        : "returning data item: $item\n") if $DEBUG;
-	return @_ ? ($self->{ data }->{ $item } = shift) 
-		  :  $self->{ data }->{ $item };
+        # get/update existing data item (must be unsealed to update)
+        if (@_ && $self->{ SEALED }) {
+            return $self->{ _CONTEXT }->throw(Template::Constants::ERROR_VIEW,
+                                  "cannot update item in sealed view: $item")
+                unless $self->{ silent };
+            # ignore args if silent
+            @_ = ();
+        }
+        $self->DEBUG(@_ ? "updating data item: $item <= $_[0]\n" 
+                        : "returning data item: $item\n") if $DEBUG;
+        return @_ ? ($self->{ data }->{ $item } = shift) 
+                  :  $self->{ data }->{ $item };
     }
     elsif (@_ && ! $self->{ SEALED }) {
-	# set data item if unsealed
-	$self->DEBUG("setting unsealed data: $item => @_\n") if $DEBUG;
-	$self->{ data }->{ $item } = shift;
+        # set data item if unsealed
+        $self->DEBUG("setting unsealed data: $item => @_\n") if $DEBUG;
+        $self->{ data }->{ $item } = shift;
     }
     elsif ($item =~ s/^$self->{ view_prefix }//) {
-	$self->DEBUG("returning view($item)\n") if $DEBUG;
-	return $self->view($item, @_);
+        $self->DEBUG("returning view($item)\n") if $DEBUG;
+        return $self->view($item, @_);
     }
     elsif ($item =~ s/^$self->{ include_prefix }//) {
-	$self->DEBUG("returning include($item)\n") if $DEBUG;
-	return $self->include($item, @_);
+        $self->DEBUG("returning include($item)\n") if $DEBUG;
+        return $self->include($item, @_);
     }
     elsif ($self->{ include_naked }) {
-	$self->DEBUG("returning naked include($item)\n") if $DEBUG;
-	return $self->include($item, @_);
+        $self->DEBUG("returning naked include($item)\n") if $DEBUG;
+        return $self->include($item, @_);
     }
     elsif ($self->{ view_naked }) {
-	$self->DEBUG("returning naked view($item)\n") if $DEBUG;
-	return $self->view($item, @_);
+        $self->DEBUG("returning naked view($item)\n") if $DEBUG;
+        return $self->view($item, @_);
     }
     else {
-	return $self->{ _CONTEXT }->throw(Template::Constants::ERROR_VIEW,
-					 "no such view member: $item");
+        return $self->{ _CONTEXT }->throw(Template::Constants::ERROR_VIEW,
+                                         "no such view member: $item");
     }
 }
 
@@ -479,8 +480,8 @@ Template::View - customised view of a template processing context
     [% VIEW view
             # some standard args
             prefix        => 'my_', 
-	    suffix        => '.tt2',
-	    notfound      => 'no_such_file'
+            suffix        => '.tt2',
+            notfound      => 'no_such_file'
             ...
 
             # any other data
@@ -508,7 +509,7 @@ Template::View - customised view of a template processing context
     [% view.header(title => 'New Title') %]
 
     # non-local templates have prefix/suffix attached
-    [% view.footer %]		# => [% INCLUDE my_footer.tt2 %]
+    [% view.footer %]           # => [% INCLUDE my_footer.tt2 %]
 
     # more verbose form of block access
     [% view.include( 'header', title => 'The Header Title' ) %]
@@ -518,7 +519,7 @@ Template::View - customised view of a template processing context
     [% view.header( title => 'The Header Title' ) %]
 
     # non-local templates have prefix/suffix attached
-    [% view.footer %]		# => [% INCLUDE my_footer.tt2 %]
+    [% view.footer %]           # => [% INCLUDE my_footer.tt2 %]
 
     # fallback on the 'notfound' template ('my_no_such_file.tt2')
     # if template not found 
@@ -529,21 +530,21 @@ Template::View - customised view of a template processing context
     # print() includes a template relevant to argument type
     [% view.print("some text") %]     # type=TEXT, template='text'
 
-    [% BLOCK my_text.tt2 %]	      # 'text' with prefix/suffix
+    [% BLOCK my_text.tt2 %]           # 'text' with prefix/suffix
        Text: [% item %]
     [% END %]
 
     # now print() a hash ref, mapped to 'hash' template
     [% view.print(some_hash_ref) %]   # type=HASH, template='hash'
 
-    [% BLOCK my_hash.tt2 %]	      # 'hash' with prefix/suffix
+    [% BLOCK my_hash.tt2 %]           # 'hash' with prefix/suffix
        hash keys: [% item.keys.sort.join(', ')
     [% END %]
 
     # now print() a list ref, mapped to 'list' template
     [% view.print(my_list_ref) %]     # type=ARRAY, template='list'
 
-    [% BLOCK my_list.tt2 %]	      # 'list' with prefix/suffix
+    [% BLOCK my_list.tt2 %]           # 'list' with prefix/suffix
        list: [% item.join(', ') %]
     [% END %]
 
@@ -585,14 +586,14 @@ second argument.
 Prefix added to all template names.
 
     [% USE view(prefix => 'my_') %]
-    [% view.view('foo', a => 20) %]	# => my_foo
+    [% view.view('foo', a => 20) %]     # => my_foo
 
 =item suffix
 
 Suffix added to all template names.
 
     [% USE view(suffix => '.tt2') %]
-    [% view.view('foo', a => 20) %]	# => foo.tt2
+    [% view.view('foo', a => 20) %]     # => foo.tt2
 
 =item map 
 
@@ -602,13 +603,13 @@ particular item.  The TEXT, HASH and ARRAY items default to 'test',
 'hash' and 'list' appropriately.
 
     [% USE view(map => { ARRAY   => 'my_list', 
-			 HASH    => 'your_hash',
-		         My::Foo => 'my_foo', } ) %]
+                         HASH    => 'your_hash',
+                         My::Foo => 'my_foo', } ) %]
 
-    [% view.print(some_text) %]		# => text
-    [% view.print(a_list) %]		# => my_list
-    [% view.print(a_hash) %]		# => your_hash
-    [% view.print(a_foo) %]		# => my_foo
+    [% view.print(some_text) %]         # => text
+    [% view.print(a_list) %]            # => my_list
+    [% view.print(a_hash) %]            # => your_hash
+    [% view.print(a_foo) %]             # => my_foo
 
     [% BLOCK text %]
        Text: [% item %]
@@ -637,22 +638,22 @@ can then make callbacks against the view to present itself.
     package Foo;
 
     sub present {
-	my ($self, $view) = @_;
-	return "a regular view of a Foo\n";
+        my ($self, $view) = @_;
+        return "a regular view of a Foo\n";
     }
 
     sub debug {
-	my ($self, $view) = @_;
-	return "a debug view of a Foo\n";
+        my ($self, $view) = @_;
+        return "a debug view of a Foo\n";
     }
 
 In a template:
 
     [% USE view %]
-    [% view.print(my_foo_object) %]	# a regular view of a Foo
+    [% view.print(my_foo_object) %]     # a regular view of a Foo
 
     [% USE view(method => 'debug') %]
-    [% view.print(my_foo_object) %]	# a debug view of a Foo
+    [% view.print(my_foo_object) %]     # a debug view of a Foo
 
 =item default
 
@@ -660,14 +661,14 @@ Default template to use if no specific map entry is found for an item.
 
     [% USE view(default => 'my_object') %]
 
-    [% view.print(objref) %]		# => my_object
+    [% view.print(objref) %]            # => my_object
 
 If no map entry or default is provided then the view will attempt to 
 construct a template name from the object class, substituting any 
 sequence of non-word characters to single underscores, e.g.
 
     # 'fubar' is an object of class Foo::Bar
-    [% view.print(fubar) %]		# => Foo_Bar
+    [% view.print(fubar) %]             # => Foo_Bar
 
 Any current prefix and suffix will be added to both the default template 
 name and any name constructed from the object class.
@@ -699,10 +700,10 @@ Prefix of methods which should be mapped to view() by AUTOLOAD.  Defaults
 to 'view_'.
 
     [% USE view %]
-    [% view.view_header() %]			# => view('header')
+    [% view.view_header() %]                    # => view('header')
 
     [% USE view(view_prefix => 'show_me_the_' %]
-    [% view.show_me_the_header() %]		# => view('header')
+    [% view.show_me_the_header() %]             # => view('header')
 
 =item view_naked
 
@@ -711,7 +712,7 @@ template names where they don't match the view_prefix.  Defaults to 0.
 
     [% USE view(view_naked => 1) %]
 
-    [% view.header() %]			# => view('header')
+    [% view.header() %]                 # => view('header')
 
 =back
 
@@ -725,18 +726,18 @@ TODO
 
 =head1 AUTHOR
 
-Andy Wardley E<lt>abw@kfs.orgE<gt>
+Andy Wardley E<lt>abw@wardley.orgE<gt> L<http://wardley.org/>
 
 =head1 COPYRIGHT
 
-Copyright (C) 2000 Andy Wardley.  All Rights Reserved.
+Copyright (C) 2000-2007 Andy Wardley.  All Rights Reserved.
 
 This module is free software; you can redistribute it and/or
 modify it under the same terms as Perl itself.
 
 =head1 SEE ALSO
 
-L<Template::Plugin|Template::Plugin>, 
+L<Template::Plugin>
 
 =cut
 
